@@ -1,5 +1,5 @@
-#This file is part of Tryton.  The COPYRIGHT file at the top level of
-#this repository contains the full copyright notices and license terms.
+# This file is part of Tryton.  The COPYRIGHT file at the top level of
+# this repository contains the full copyright notices and license terms.
 import uuid
 import vobject
 import dateutil.tz
@@ -11,7 +11,7 @@ from sql import Table, Column
 from trytond.model import ModelSQL, ModelView, fields
 from trytond.tools import reduce_ids
 from trytond import backend
-from trytond.pyson import Eval, If, Bool
+from trytond.pyson import Eval, If, Bool, PYSONEncoder
 from trytond.transaction import Transaction
 from trytond.pool import Pool
 from trytond.modules.calendar import AlarmMixin, DateMixin, RRuleMixin, \
@@ -107,19 +107,13 @@ class Todo(ModelSQL, ModelView):
             states={
                 'invisible': Bool(Eval('parent')),
             }, depends=['parent'])
-    calendar_owner = fields.Function(fields.Many2One('res.user', 'Owner'),
-            'get_calendar_field', searcher='search_calendar_field')
-    calendar_read_users = fields.Function(fields.One2Many('res.user', None,
-        'Read Users'), 'get_calendar_field', searcher='search_calendar_field')
-    calendar_write_users = fields.Function(fields.One2Many('res.user', None,
-        'Write Users'), 'get_calendar_field', searcher='search_calendar_field')
     vtodo = fields.Binary('vtodo')
 
     @classmethod
     def __setup__(cls):
         super(Todo, cls).__setup__()
         cls._sql_constraints = [
-            #XXX should be unique across all componenets
+            # XXX should be unique across all componenets
             ('uuid_recurrence_uniq', 'UNIQUE(uuid, calendar, recurrence)',
                 'UUID and recurrence must be unique in a calendar.'),
             ]
@@ -133,13 +127,14 @@ class Todo(ModelSQL, ModelView):
         # Migrate from 1.4: remove classification_public
         ModelData = pool.get('ir.model.data')
         Rule = pool.get('ir.rule')
-        models_data = ModelData.search([
-            ('fs_id', '=', 'rule_group_read_todo_line3'),
-            ('module', '=', module_name),
-            ], limit=1)
-        if models_data:
-            model_data, = models_data
-            Rule.delete([Rule(model_data.db_id)])
+        with Transaction().set_user(0):
+            models_data = ModelData.search([
+                ('fs_id', '=', 'rule_group_read_todo_line3'),
+                ('module', '=', module_name),
+                ], limit=1)
+            if models_data:
+                model_data, = models_data
+                Rule.delete([Rule(model_data.db_id)])
         super(Todo, cls).__register__(module_name)
 
     @staticmethod
@@ -160,31 +155,17 @@ class Todo(ModelSQL, ModelView):
 
     @fields.depends('status', 'completed', 'percent_complete')
     def on_change_status(self):
-        res = {}
-        if not getattr(self, 'status', None):
-            return res
+        if not self.status:
+            return
         if self.status == 'completed':
-            res['percent_complete'] = 100
-            if not getattr(self, 'completed', None):
-                res['completed'] = datetime.datetime.now()
-
-        return res
+            self.percent_complete = 100
+            if not self.completed:
+                self.completed = datetime.datetime.now()
 
     @staticmethod
     def timezones():
         return [(x, x) for x in pytz.common_timezones] + [('', '')]
 
-    def get_calendar_field(self, name):
-        name = name[9:]
-        if name in ('read_users', 'write_users'):
-            return [x.id for x in getattr(self.calendar, name)]
-        else:
-            return getattr(self.calendar, name).id
-
-    @classmethod
-    def search_calendar_field(cls, name, clause):
-        return [('calendar.' + name[9:],) + tuple(clause[1:])]
-
     @classmethod
     def validate(cls, todos):
         super(Todo, cls).validate(todos)
@@ -204,6 +185,12 @@ class Todo(ModelSQL, ModelView):
                 or self.occurences):
             self.raise_user_error('invalid_recurrence', (self.rec_name,))
 
+    @classmethod
+    def view_attributes(cls):
+        return [('//page[@id="occurences"]', 'states', {
+                    'invisible': Bool(Eval('_parent_parent')),
+                    })]
+
     @classmethod
     def create(cls, vlist):
         pool = Pool()
@@ -226,7 +213,7 @@ class Todo(ModelSQL, ModelView):
                             if x.status != 'declined'
                             and x.email != todo.parent.organizer]
                 if attendee_emails:
-                    with Transaction().set_context(_check_access=False):
+                    with Transaction().set_user(0):
                         calendars = Calendar.search([
                             ('owner.email', 'in', attendee_emails),
                             ])
@@ -332,24 +319,27 @@ class Todo(ModelSQL, ModelView):
                                 if x.status != 'declined'
                                 and x.email != todo.organizer]
                     else:
-                        attendee_emails = [x.email for x in todo.parent.attendees
-                                if x.status != 'declined'
-                                and x.email != todo.parent.organizer]
+                        attendee_emails = [
+                            x.email for x in todo.parent.attendees
+                            if x.status != 'declined'
+                            and x.email != todo.parent.organizer]
                     if attendee_emails:
-                        with Transaction().set_context(_check_access=False):
+                        with Transaction().set_user(0):
                             todo2s = cls.search([
-                                ('uuid', '=', todo.uuid),
-                                ('calendar.owner.email', 'in', attendee_emails),
-                                ('id', '!=', todo.id),
-                                ('recurrence', '=', todo.recurrence),
-                                ])
+                                    ('uuid', '=', todo.uuid),
+                                    ('calendar.owner.email', 'in',
+                                        attendee_emails),
+                                    ('id', '!=', todo.id),
+                                    ('recurrence', '=', todo.recurrence),
+                                    ])
                         for todo2 in todo2s:
                             if todo2.calendar.owner.email in attendee_emails:
-                                attendee_emails.remove(todo2.calendar.owner.email)
-                        with Transaction().set_context(_check_access=False):
+                                attendee_emails.remove(
+                                    todo2.calendar.owner.email)
+                        with Transaction().set_user(0):
                             cls.write(todos, todo._todo2update())
                     if attendee_emails:
-                        with Transaction().set_context(_check_access=False):
+                        with Transaction().set_user(0):
                             calendars = Calendar.search([
                                 ('owner.email', 'in', attendee_emails),
                                 ])
@@ -402,7 +392,7 @@ class Todo(ModelSQL, ModelView):
                     attendee_emails = [x.email for x in todo.parent.attendees
                             if x.email != todo.parent.organizer]
                 if attendee_emails:
-                    with Transaction().set_context(_check_access=False):
+                    with Transaction().set_user(0):
                         todos_delete = cls.search([
                             ('uuid', '=', todo.uuid),
                             ('calendar.owner.email', 'in', attendee_emails),
@@ -416,7 +406,7 @@ class Todo(ModelSQL, ModelView):
                     organizer = todo.organizer
                 else:
                     organizer = todo.parent.organizer
-                with Transaction().set_context(_check_access=False):
+                with Transaction().set_user(0):
                     todo2s = cls.search([
                         ('uuid', '=', todo.uuid),
                         ('calendar.owner.email', '=', organizer),
@@ -1043,8 +1033,8 @@ class TodoAttendee(AttendeeMixin, ModelSQL, ModelView):
                     values=[attendee.select(attendee.email,
                             where=attendee.id == sql_table.calendar_attendee),
                         attendee.select(attendee.status,
-                            where=
-                            attendee.id == sql_table.calendar_attendee)]))
+                            where=(
+                                attendee.id == sql_table.calendar_attendee))]))
             table.drop_column('calendar_attendee', True)
 
     @classmethod
@@ -1073,7 +1063,7 @@ class TodoAttendee(AttendeeMixin, ModelSQL, ModelView):
                     attendee_emails = [x.email for x in todo.parent.attendees
                             if x.email != todo.parent.organizer]
                 if attendee_emails:
-                    with Transaction().set_context(_check_access=False):
+                    with Transaction().set_user(0):
                         todos = Todo.search([
                             ('uuid', '=', todo.uuid),
                             ('calendar.owner.email', 'in', attendee_emails),
@@ -1122,7 +1112,7 @@ class TodoAttendee(AttendeeMixin, ModelSQL, ModelView):
                     attendee_emails = [x.email for x in todo.parent.attendees
                             if x.email != todo.parent.organizer]
                 if attendee_emails:
-                    with Transaction().set_context(_check_access=False):
+                    with Transaction().set_user(0):
                         attendees2 = cls.search([
                                 ('todo.uuid', '=', todo.uuid),
                                 ('todo.calendar.owner.email', 'in',
@@ -1157,7 +1147,7 @@ class TodoAttendee(AttendeeMixin, ModelSQL, ModelView):
                     attendee_emails = [x.email for x in todo.attendees
                             if x.email != todo.parent.organizer]
                 if attendee_emails:
-                    with Transaction().set_context(_check_access=False):
+                    with Transaction().set_user(0):
                         attendees = cls.search([
                             ('todo.uuid', '=', todo.uuid),
                             ('todo.calendar.owner.email', 'in',
@@ -1175,7 +1165,7 @@ class TodoAttendee(AttendeeMixin, ModelSQL, ModelView):
                     organizer = todo.organizer
                 else:
                     organizer = todo.parent.organizer
-                with Transaction().set_context(_check_access=False):
+                with Transaction().set_user(0):
                     attendees = cls.search([
                         ('todo.uuid', '=', todo.uuid),
                         ('todo.calendar.owner.email', '=', organizer),
